package tasks;

import java.io.File;
import java.text.NumberFormat;
import java.util.Vector;

import org.apache.commons.math3.stat.inference.ChiSquareTest;

/**
 * Der Chi-Quadrat-Test für pgm-Bilder
 * @author sc2
 */

public class ChiTest {

    /**
     * Liest ein .pgm-Bild aus der angegebenen Datei ein, teilt dieses in 100 Teile ein,
     * und gibt für jeden Teil die Einbettungswahrscheinlichkeit nach dem Chi-Square-Test
     * aus
     * @param file die Bilddatei
     */
	
    public static String Chitest(File file) {
        String output = "";
    	
        //read the file
        HelperFunctions.Image image = null;
        try {
        	image = HelperFunctions.read(file);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            System.exit(1);
        }

        //cut picture into 100 pieces
        for (int k = 1; k <= 100; k++) {
            //create histogram
            int[] frequency = new int[256];
            for (int h = 0; h < 256; h++) {
                frequency[h] = 0;
            }

            //percentage of pixels to use for one piece
            int proz = (int) ((float) ((float)k * (float)image.getSize()) / 100.0);
            for (int h = 0; h < proz; h++) {
                int i = image.getData()[h];
                frequency[i]++;
            }

            Vector<Double> expectedVec = new Vector<Double>();
            Vector<Long> realVec = new Vector<Long>();
            for (int i = 0; i <= 127; i++) {
            	// average between even and odd gray value
            	double e = (double) ((float)(frequency[2* i] + frequency[(2* i) + 1]) / 2.0);
            	// only consider values > 4 (according to the original paper of Andreas Westfeld)
            	if(e > 4) {
                    //fill array of expected values 
            		expectedVec.add(e);
            		//fill array of real values with even gray values
            		realVec.add((long)frequency[2*i]);
            	}
            }
            double expected[] = new double[expectedVec.size()];
            for(int j = 0; j < expectedVec.size(); j++) {
            	expected[j] = expectedVec.elementAt(j).doubleValue();
            }
            long real[] = new long[realVec.size()];
            for(int j = 0; j < realVec.size(); j++) {
            	real[j] = realVec.elementAt(j).longValue();
            }
                        
            double result;
            if(expected.length < 2) {
            	result = 0.0;
            } else {
                // Use a verified implementation...
                ChiSquareTest cst = new ChiSquareTest();
            	result = cst.chiSquareTest(expected, real);

            	//or the old version 
              //  result = chiSquareTest(real, expected);
            }
            
            //return results
        /*    if (result < 0.0) {
                result = 0.0;
            } else if (result > 1.0) {
                result = 1.0;
            }
            */
            NumberFormat nf = NumberFormat.getInstance();
            nf.setMaximumFractionDigits(3);
            nf.setMinimumFractionDigits(3);

            
            
            output = output + tasks.Main.messages.getString("chiTest_for") + " "
            + ((k < 10) ? " " : "")
            + ((k < 100) ? " " : "") + k + tasks.Main.messages.getString("chiTest_ofHundredOfThePic") + " "
            + nf.format(result) + " " + tasks.Main.messages.getString("chiTest_embeddingProbability") + "\n";
        }
        return output;
    }

    /**
     * Chi-Quadrat-Test
     * @author sc2, frei nach Thomas Pucklitzsch
     * @param org Verteilung der Farbwerte im Bild
     * @param expected erwartete Farbwerteverteilung
     * @return chi-square
     */
    
    static double chiSquareTest(
            final long[] org,
            final double [] expected) {
        double X2 = 0.0;
        for (int  i = 0; i < org.length; i++) {
        	X2 += ((double)((org[i] - expected[i]) * (org[i] - expected[i]))) / (double)expected[i];
        }
        double degreeCoefficient = Math.pow(2.0, (double)org.length / 2.0) * gammaOfHalf(org.length);
        double degreeExponent = (double)org.length / 2.0 - 1.0;

        X2 = numericIntegral(0.0, X2, 000.1, degreeCoefficient, degreeExponent);
        return (1 - X2);
    }


    /**
    /* this is the implementation of X square function
     * @param x x
     * @param degreeCoefficient unknown
     * @param degreeExponent unknown
     * @return X2
     */
    private static double function(
            final double x,
            final double degreeCoefficient,
            final double degreeExponent) {
        double result;
        result = Math.pow(x, degreeExponent);
        result *=  Math.exp(-x / 2.0) / degreeCoefficient;
        return result;
     }

    /**
     * Gamma of half
     * @param i unclear
     * @return gamma
     */
    private static double gammaOfHalf(final int i) {
        double gamma, d;
        int j;
        if ((i & 1) == 0) {
            //i gerade
            gamma = 1;
            for (j = 2; j < i / 2; j++) {
                gamma *= j;
            }
        } else {
            d = i / 2.0 - 1.0;
            gamma = Math.sqrt(22 / 7);
            for (j = 0; j < i / 2; j++) {
                gamma *= d;
                d -= 1.0;
            }
        }
        return gamma;
    }


    /**
     * computes the integral of the interval from a to b
     * @param a a
     * @param b b
     * @param size size
     * @param degreeCoefficient unknown
     * @param degreeExponent unknown
     * @return the integral
     */
    static double numericIntegral(
            final double a,
            final double b,
            final double size,
            final double degreeCoefficient,
            final double degreeExponent) {
        double area = 0.0;

        double aTemp = a + size / 2;
        while (aTemp < b) {
            area += size * function(aTemp, degreeCoefficient, degreeExponent);
            aTemp += size;
        }
        double sizeTemp = aTemp - b;
        area -= sizeTemp * function(b + (sizeTemp / 2), degreeCoefficient, degreeExponent);
     return area;
     }

}
