package tasks;

/**
 * Abstrakte Klasse für die Implementierung eines Stego-Algorithmus.
 * @author sc2
 *
 */

public abstract class EmbeddingAlgorithm {

	double embeddingRate;
	double shrinkageRate;
	double alterationRate;
	double capacity;

	public EmbeddingAlgorithm() {
		super();
	}

	/**
	 * Daten in ein Bild einbetten. Abgeleitete Klassen müssen dafür einen Algorithmus implementieren.
	 * @param cover das Trägermedium
	 * @param percent Anteil des Bildes, der für die Einbettung genutzt werden soll
	 * @param emb die einzubettende Nachricht
	 * @param key Schlüssel, Blockgröße etc. für den Algorithmus
	 * @return das Steganogramm
	 */
	
	public abstract HelperFunctions.Image embed(HelperFunctions.Image cover, int percent, boolean[] emb, int[] key);
	
	/**
	 * Auslesen einer Nachricht aus einem Steganogramm. Abgeleitete Klassen müssen hier einen Algorithmus
	 * implementieren.
	 * @param stego das Steganogramm
	 * @param key Schlüssel, Blockgröße etc. für den Algorithmus
	 * @return die ausgelesene Nachricht
	 */
	
	public abstract boolean[] extract(HelperFunctions.Image stego, int[] key);


	/**
	 * Gibt die Änderungsrate zurück.
	 * @return die Änderungsrate
	 */
	
	public String getAlterationRate() {
		
		String x = Double.toString(alterationRate);
		if(x.length()>5)
		x = x.subSequence(0,5).toString() + "%";
		else x= x + "%";
		return x;
	}

	/**
	 * Gibt die Einbettungsrate zurück.
	 * @return die Einbettungsrate
	 */
	
	public String getEmbeddingRate() {
		String x = Double.toString(embeddingRate);
		if(x.length()>5)
			x = x.subSequence(0,5).toString() + "%";
			else x= x + "%";
		return x;
	}

	/**
	 * Gibt die Einbettungskapazität zurück.
	 * @return die Einbettungskapazität
	 */
	
	public String getCapacity() {
		String x = Double.toString(capacity);
		if(x.length()>5)
			x = x.subSequence(0,5).toString() + "%";
			else x= x + "%";
		return x;
	}

	/**
	 * Gibt die Schwundrate zurück.
	 * @return die Schwundrate
	 */
	
	public String getShrinkageRate() {
		String x = Double.toString(shrinkageRate);
		if(x.length()>5)
			x = x.subSequence(0,5).toString() + "%";
			else x= x + "%";
		return x;
	}

}